{% import 'system_macros.html' as system %}

<html lang="en">
     <head>
{% block head %}
{{system.head(data=page)}}
<!--  CSS -->
<link rel="stylesheet" href="{{url_for('static')}}/css/pencil.css">
<link rel="preconnect" href="https://fonts.googleapis.com">
<link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
<link href="https://fonts.googleapis.com/css2?family=Courier+Prime:ital,wght@0,400;0,700;1,400;1,700&display=swap" rel="stylesheet">
<link href="https://fonts.googleapis.com/css2?family=Courier+Prime:wght@400;700&family=Hanken+Grotesk:ital,wght@0,100..900;1,100..900&display=swap" rel="stylesheet">

{% endblock %}
     </head>

<body class="courier-prime-regular">
    <header id="header" class="row">
    {% block header_nav %}
        <nav id="navigation" class="menu inline">
            <ul class="menu center">
                    <li class="active">
                        <a href="/">Pyonir</a>
                    </li>
            </ul>
        </nav>
    {% endblock %}
    </header>

    <div id="content" class="row">
        <div class="content-body">
        {% block content %}
        <h1 style="text-align: center;">{{page.title}}</h1>
        {{ page.content | safe }}
        {% endblock %}
        </div>
    </div>


    <footer>
        <div class="credits text-center">Pencil Theme Powered by Pyonir</div>
    </footer>
</body>

{% block js %}
{% yield 'js' %}
{% endblock %}

</html>