<!-- Essential Meta Tags -->
<meta charset="UTF-8" />
<meta name="viewport" content="width=device-width, initial-scale=1.0" />
<title>Your Page Title Here</title>
<meta name="description" content="A concise, keyword-rich summary of your page." />
<meta name="keywords" content="primary keyword, secondary keyword, brand name" />
<meta name="author" content="Your Name or Company" />
<meta name="robots" content="index, follow" />

<!-- Canonical URL -->
<link rel="canonical" href="https://www.example.com/your-page-url/" />

<!-- Open Graph (for Facebook, LinkedIn, etc.) -->
<meta property="og:type" content="website" />
<meta property="og:title" content="Your Page Title Here" />
<meta property="og:description" content="Same as meta description, or slightly extended." />
<meta property="og:image" content="https://www.example.com/image.jpg" />
<meta property="og:url" content="https://www.example.com/your-page-url/" />
<meta property="og:site_name" content="Your Site Name" />

<!-- Twitter Card Tags -->
<meta name="twitter:card" content="summary_large_image" />
<meta name="twitter:title" content="Your Page Title Here" />
<meta name="twitter:description" content="Same as meta description." />
<meta name="twitter:image" content="https://www.example.com/image.jpg" />
<meta name="twitter:site" content="@YourTwitterHandle" />

<!-- Favicons & App Icons -->
<link rel="icon" href="/favicon.ico" />
<link rel="apple-touch-icon" href="/apple-touch-icon.png" />
<link rel="manifest" href="/site.webmanifest" />

<!-- Schema.org Structured Data (JSON-LD) -->
<script type="application/ld+json">
{
"@context": "https://schema.org",
"@type": "WebSite",
"name": "Your Site Name",
"url": "https://www.example.com/"
}
</script>

<!-- Preconnect for performance -->
<link rel="preconnect" href="https://fonts.googleapis.com" />
<link rel="preconnect" href="https://fonts.gstatic.com" crossorigin />

<!-- Optional: Google Fonts or other styles -->
<link href="https://fonts.googleapis.com/css2?family=Inter&display=swap" rel="stylesheet" />

