<script type="application/ld+json"> {"@context": "http://schema.org", "@type": "WebSite",
  "name": "{{ configs.name|safe }}",
  "description": "{{ configs.description|safe }}",
  "url":"{{ request.protocol }}{{ request.host }}{{ request.url }}"
}</script>