@import "reset.css";
@import "typography.css";
@import "grid.css";
@import "forms.css";
@import "menus.css";

body, html {
    overscroll-behavior-y: none; /* Prevents both vertical and horizontal overscroll */
}