{% extends 'base.html' %}
{% block content %}
<div style="text-align: center;">
    <h1>{{request.status_code}} Error!</h1>
    <em>{{page.content}}</em>
    {{request.notifications}}
</div>
{% endblock %}